_copyright="""
Copyright 2007 Tasci Synx

    This file is part of imageTagger.

    imageTagger is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    imageTagger is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from thumb import thumb
import spool
spool = spool.Spool()

import sqlalchemy as db
from datetime import datetime

import base64,sha
import urllib2
from datetime import datetime

#	conn = metadata.engine.connect()
#	trans = conn.begin()
#	conn.execute("""CREATE RULE recentlimit AS ON INSERT TO recent_tags DO ALSO
#	 DELETE FROM recent_tags WHERE id NOT IN
#	   (SELECT id FROM recent_tags ORDER BY id DESC LIMIT 10);""")
#	trans.commit()

 # class definitions
class Tag(object):
	def __init__(self, name):
		self.name = name
		self.uses = 0
	def __repr__(self):
		return "Tag(%r)" % self.name
 
class Error(Exception): pass

def download():
 #cookiejar = cookiejar.MozillaCookieJar( XXX: how to find Mozilla/etc cookies?
 # ...how to avoid downloading the image twice? XXX
 # idea: have this program proxy for the browser?
 # idea: proxy fills up a spool, this function checks
 proxy = urllib2.ProxyHandler({'http': 'http://localhost:3128'})
 cookiejar = urllib2.ProxyHandler({'http': 'http://localhost:3128'})
 _opener = urllib2.build_opener(proxy)
 def process(origin):
 	gotIt = False
 	try:
		type,data = spool.load(origin)
		print "Heh, hah! Spooled",origin
 		gotIt = True
	except IOError: pass
	if not gotIt:
		# Then we have to download it again... and pray
	 	request = urllib2.Request(origin,headers={'User-Agent': 'synxes swallow not spit'})
 		try: res = _opener.open(request)
		except urllib2.HTTPError,e:
			raise Error("Could not download %s because of error. %s" % (origin,e))

	 	type = res.headers["Content-type"]
	 	if type.startswith('text/'):
	 		raise Error("We don't tag text %s %s" % (type,origin))
	 	data = res.read()
	hash = base64.b64encode(sha.new(data).digest())
	return type,hash,data
 process.__name__ = download.__name__
 return process
download = download()

class Image(object):
	tags = []
	data = ''
 	def __init__(self, origin, type, hash, data):
		self.origin = origin
		self.type = type
		self.hash = hash
		self.data = data
		self.thumbnail = thumb(self.data,self.type)
	def setTags(self,db,tags):
		self.tags = tags
		db.session.save(self)
		db.session.flush()
		for tag in tags:
			tag.modified = datetime.now()
			tag.uses = len(tag.images) # for ORDER BY support grumble...
			db.session.save(tag)
		db.session.flush()
	def __repr__(self):
		return 'Image(%r)' % self.origin

def singleton(thing):
	global _item
	try:
		return _item
	except NameError:
		_item = thing

class singleton:
	def __init__(self,cls):
		self.instance = None
		self.cls = cls
	def __call__(self,*args,**kw):
		if self.instance: return self.instance
		self.instance = self.cls(*args,**kw)
		return self.instance

class attrdict(dict):
	def __getattr__(self,name):
		return self[name]
	def __setattr__(self,name,value):
		self[name] = value

class Database:
	def cleanTags(self,tags):
		for tag in tags:
			tag.uses = len(tag.images)
			print "Cleaning",tag.name,tag.uses
			if tag.uses <= 0:
				self.session.delete(tag)
		self.session.flush()

	def create(self):
		metadata = self.metadata
		self.tables = attrdict()
		from sqlalchemy import *
		self.tables.images = Table('images', metadata,
			Column('id', Integer, primary_key=True),
			Column('hash', String(28), unique=True),
			Column('origin', TEXT, unique=True),
			Column('type', TEXT),
			Column('thumbnail', Binary()),
			Column('data', Binary())
		)
		
		self.tables.tags = Table('tags', metadata,
			Column('id', Integer, primary_key = True),
			Column('name', String(50), unique = True),
			Column('modified', DateTime),
			Column('uses', Integer)
		)
		
		self.tables.itemtags = Table('image_tags', metadata,
			Column('image', Integer, ForeignKey("images.id")),
			Column('tag', Integer, ForeignKey("tags.id")),
			UniqueConstraint('image', 'tag')
		)
	
		metadata.create_all()

#		conn = metadata.engine.connect()
#		trans = conn.begin()
#		conn.execute("""ALTER TABLE images ADD COLUMN modified DateTime;""")
#		trans.commit()
#		raise SystemExit
#

	def __init__(self,location='sqlite://./tags.db'):
		self.location = location
		self.metadata = db.BoundMetaData(location)
		#self.metadata.engine.echo = True
		self.create()
		
		db.mapper(Image, self.tables.images, properties = {
			'tags':db.relation(Tag, secondary=self.tables.itemtags, lazy=False),
			'thumbnail':db.deferred(self.tables.images.c.thumbnail),
			'data':db.deferred(self.tables.images.c.data),
			}
		)

		db.mapper(Tag, self.tables.tags, properties = {
			'images':db.relation(Image, secondary=self.tables.itemtags, lazy=True)
			}
		)

		self.session = db.create_session()
	def recent(self,howMany=5):
		query = self.session.query(Tag)
		for tag in query.select(order_by=Tag.c.modified):
			yield tag
			howMany -= 1
			if howMany <= 0: break
	def popular(self,howMany=5):
		query = self.session.query(Tag)
		for tag in query.select(order_by=db.desc(Tag.c.uses)):
			yield tag
			howMany -= 1
			if howMany <= 0: break
	def findTag(self,name):
	 	item = self.session.query(Tag).selectfirst(Tag.c.name==name)
	 	if not item: 
	 		item = Tag(name)
	 	return item
	def findImageByOrigin(self,origin,create=True):
		image = self.session.query(Image).selectfirst(Image.c.origin==origin)
	 	if not image:
			if not create: return None
	 		type,hash,data = download(origin)
	 		image = self.session.query(Image).selectfirst(Image.c.hash==hash)
	 		if image:
				print "Warning, %s already found as %s" % (origin,image.origin)
	 		else:
				print "Creating Image",origin
		 		image = Image(origin,type,hash,data)
	 			self.session.save(image)
				self.session.flush()
	 	return image
	def record(self,url,tags):
		for name in tags:
			tag = self.findTag(name)
			image = self.findImageByOrigin(url)
			tags = set(image.tags)
			tags.add(tag)
			image.setTags(self,tags)
			self.session.save(image)
			self.session.flush()
Database = singleton(Database)

open = Database

if __name__=='__main__':
	mydb = Database()

