_copyright="""
Copyright 2007 Tasci Synx

    This file is part of imageTagger.

    imageTagger is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    imageTagger is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""



"""Many files have BAD names. Often % will be substituted with %25, and then %25 to %2525 and so on.  Also there's a sort of system among Japanese that's like #xxxxx; for a unicode character. Which gets converted to %23xxxxx%3B since # and ; are illegal in url paths. Pretty much it's a horrible system, and filesystems are much nicer, especially ones like mine that can support any unicode character. So here's my best attempt to de-derange these files."""

import os
import shutil
import urllib
import re,struct
from itertools import count

def fixpercent(badname):
	if not '%' in badname: return badname
	return urllib.unquote(badname) 
	# urllib.unquote can't handle the %25%23xxxxx%3b misconvention
	
def fixwapanese(badname):
	if not ( '#' in badname and ';' in badname ): return badname
	num = re.compile('#([0-9]+);')
	def doit(match):
		num = int(match.group(1))
		bytes = struct.pack('h',num)
		for charset in ['utf8','utf16']:
		 try:
			return unicode(bytes,charset)
		 except UnicodeDecodeError: pass
		raise RuntimeError("Could not decode %s" % bytes)
	goodname = num.sub(doit,badname)
	return goodname

def fixchan():
	chanstamp = re.compile('(?:[a-z]*_|)[0-9]{10,13}[._]?')
	def process(badname):
		if os.path.islink(badname): return badname
		goodname = chanstamp.sub('',badname)
		if goodname == badname: return badname
		
		if goodname.startswith('./'): goodname = goodname[2:]
		top,base = os.path.split(goodname)
		if base.startswith('.'): 
			base = base[1:]
			goodname = os.path.join(top,base)
		if not '.' in base:
			return badname
		return goodname
	return process
fixchan = fixchan()

def fixendcaps(badname):
	where = badname.rfind('.')
	if where == -1: return badname
	return badname[:where]+badname[where:].lower()

fixes = (fixpercent,fixwapanese,fixchan,fixendcaps)

def fix(badname):
	goodname = badname
	for f in fixes:
		goodname = f(goodname)
	return goodname

def main():
	for top,dirs,files in os.walk('.'):
		for badname in files:
			badname = os.path.join(top,badname)
			goodname = fix(badname)
			if goodname != badname:
				print "%s -> %s" % (badname,goodname)
				os.rename(badname,goodname)

if __name__=='__main__': main()

