_copyright="""
Copyright 2007 Tasci Synx

    This file is part of imageTagger.

    imageTagger is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    imageTagger is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import gtk
from gtk import glade

import os,sys,signal,time
import tempfile
from subprocess import call,Popen
from weakref import WeakValueDictionary

from myurl import URL
import database
from thumb import thumb

from resources import resource
from isImage import is_image

TARGET_TYPE_TEXT = 80
TARGET_TYPE_PIXMAP = 81
TARGET_TYPE_JPEG = 71

def cleanOut(widget):
	widget.foreach(lambda child: widget.remove(child))


#types = [
#	'text/unicode',
#	'application/x-moz-nativeimage',
#	'application/x-moz-file-promise',
#	'text/x-moz-url',
#	'text/html',
#	'_NETSCAPE_URL',
#	'text/x-moz-url-data',
#	'text/x-moz-url-desc',
#	'text/_moz_htmlcontext',
#	'text/_moz_htmlinfo',
#	'text/plain',
#	'application/x-moz-file-promise-url',
#	'application/x-moz-file-promise-dest-filename',
#] # yanked from debug drop_drag below

types = ['text/unicode','text/plain']

def tagsplit(s):
	if ',' in s:
		return set((piece.strip() for piece in s.split(',')))
	else:
		return set((piece for piece in s.split(' ') if len(piece)))
class Entry:
	_fields_ = (
			'locationDest',
			'tagsEntry',
			'recentContainer',
			'popularContainer',
			'cancelButton',
			'submitButton',
			'testButton')
	def __init__(self,factory):
		self.dialog = factory.xml.get_widget('inputTags')
		for name in self._fields_:
			setattr(self,name,factory.xml.get_widget(name))
		self.destroyed = False
	def destroy(self):
		self.dialog.destroy()
		del self.dialog
		for attr in self._fields_:
			delattr(self,attr)
		self.destroyed = True


class EntryFactory:
	def __init__(self): pass
	def __call__(self):
		self.xml = glade.XML(resource("entry.glade"),'inputTags')
		return Entry(self)

class Tagger:
	def submit(self,ui):
		location = ui.locationDest.get_text()
		tags = ui.tagsEntry.get_text()
		ui.destroy()

		if not len(location): return

		#XXX
		#tags = 'cute,furry,sexy,canine,dog,cartoon'
		#location='file:///home/pandora/art/yiffy/fchan/rain/3princesses.gif'
		url = URL(location)
		if url.scheme == 'http':
			print "HTTP",url
		elif ( url.scheme == 'file' or url.scheme == '' ) and os.path.exists(url.path):
			url.scheme = 'file'
			print "Direct link to",url

		tags = set(self.buttonTags).union(tagsplit(tags))
		if tags.issubset(self.oldTags):
			print "No new tags"
		else:
			#tags = tuple(tags)
			print "Tags: ",', '.join(tags)
			try:
				self.db.record(unicode(url),tags)
			except database.Error,e:
				print e
				print "Could not save tags"
	def receive_drag(self,widget,context,x,y,selection,info,ttime):
		if selection.type.startswith('text'):
			for codec in ('utf16','utf8','latin1'):
				try:
					origin = unicode(selection.data,codec)
					break
				except UnicodeDecodeError: pass
			self.createEntry(origin)
		else:
			print selection.type,selection.data
			origin = selection.get_text()
			context.finish(True,False,int(time.time()))
			raise SystemExit
	def createEntry(self,origin):
		if origin.find(' ')!=-1:
			return
		url = URL(origin)
		if url.netloc in ('synx.us.to','synx.dyndns.org','63.197.122.98'):
			if url.path.startswith('/art'):
				url.path = '/no' + url.path[1:]
		if not is_image(os.path.basename(url.path)):
			print "Origin (%s)" % os.path.basename(url.path)
			return
		origin = unicode(url)

		try:
			if memory.has_key(origin): 
				ui = memory[origin]
				if ui.destroyed: 
					del memory[origin]
				else:
					win = memory[origin].dialog
					win.hide()
					win.show_all()
					win.grab_focus()
					return
		except NameError:
			print "Init memory"
			memory = WeakValueDictionary()

		ui = self.makeDialog()
		memory[origin] = ui

		self.oldTags = set()
		ui.locationDest.set_text(origin)
		image = self.db.findImageByOrigin(origin,create=False)
		if image and len(image.tags):
			self.oldTags = set([t.name for t in image.tags])
			ui.tagsEntry.set_text(', '.join(self.oldTags))

		self.fillTags(self.db.recent(),ui.recentContainer)
		self.fillTags(self.db.recent(),ui.popularContainer)

		cancelButton = ui.cancelButton
		cancelButton.connect("clicked",lambda btn: ui.destroy())
		submitButton = ui.submitButton
		submitButton.connect("clicked",lambda btn: self.submit(ui))
		ui.testButton.connect("clicked",lambda btn: self.test(ui))
		ui.tagsEntry.grab_focus()
		ui.dialog.show_all()
	def fillTags(self,tags,container):
		self.buttonTags = set()
		def add(btn,tag):
			if btn.get_active():
				self.buttonTags.add(tag)
			else:
				self.buttonTags.discard(tag)
		cleanOut(container)
		for tag in tags:
			button = gtk.CheckButton(label=tag.name)
			button.connect("clicked",add,tag.name)
			container.pack_start(button)
		container.show_all()
	def test(self,ui):
		tag = self.db.findTag("synx")
		for image in tag.images:
			print image.origin
			if not 'eevee' in set(t.name for t in image.tags):
				continue
			out = tempfile.NamedTemporaryFile()
			out.write(image.thumbnail)
			out.flush()
			call(['display',out.name])
			out.close()

	makeDialog = EntryFactory()
	def __init__(self,db):
		self.db = db

def reapChild(number,frame):
	try:
		while True:
			os.waitpid(-1,0)
	except OSError: pass
	signal.signal(signal.SIGCHLD,reapChild) # loathe SysV

signal.signal(signal.SIGCHLD,reapChild)

class SelectDisplay:
	def __init__(self,location):
		self.on = False
		self.location = location
	def __call__(self,*args):
		if self.on:
			self.on = False
			try:
				os.kill(self.process.pid,signal.SIGTERM)
			except OSError: 
				# Process already died
				pass
			self.process = None
		else:
			self.on = True
			self.process = Popen(['python','selectImages.py',self.location])

class MouseGrabber:
	def __init__(self,w,cb):
		self.w = w
		self.callback = cb
		w.grab_add()
		self.mover = w.connect('motion-notify-event',self.move)
		self.release = w.connect('button-release-event',lambda *args: self.stopMoving())
	def move(self,window,event):
		print "Moving!"
		self.w.move(event.x_root,event.y_root)
	def stopMoving(self):
		self.w.grab_remove()
		self.w.disconnect(self.mover)
		self.w.disconnect(self.release)

if __name__=='__main__':
	dbname = sys.argv[1]
	if not dbname.startswith('sqlite:'):
		dbname = 'sqlite:///'+dbname
	print dbname
	db = database.open(dbname)
	icon = gtk.gdk.pixbuf_new_from_file(resource("icon.png"))
	gtk.window_set_default_icon(icon)
	icon = icon.scale_simple(16,16,gtk.gdk.INTERP_BILINEAR)
	app = Tagger(db)

	xml = glade.XML(resource("target.glade"),'top')
	top = xml.get_widget('top')
	event = xml.get_widget('event')
	image = xml.get_widget('image')

	image.set_from_pixbuf(icon)
	event.connect("button-press-event",lambda *args: MouseGrabber(top,lambda: None))

	accept = [ ( "text/plain", 0, TARGET_TYPE_TEXT ),
		( "image/x-xpixmap", 0, TARGET_TYPE_PIXMAP ) ]

	# XXX: How to get the URL/origin if just an image is sent over?
	accept = list()
	for num in range(len(types)):
		type = types[num]
		accept.append((type,0,num))

	#accept = [ ( "*", 0, TARGET_TYPE_JPEG ) ]

	action = gtk.gdk.ACTION_DEFAULT | gtk.gdk.ACTION_COPY | gtk.gdk.ACTION_MOVE | gtk.gdk.ACTION_LINK | gtk.gdk.ACTION_PRIVATE | gtk.gdk.ACTION_ASK

	event.connect("drag-data-received",app.receive_drag)
	event.drag_dest_set(gtk.DEST_DEFAULT_ALL,
		accept, action)

	# Use this to debug drag types:
	#def drop_drag(widget,context,x,y,ttime):
	#	print context.targets
	#	raise SystemExit
	#event.connect("drag-drop",app.drop_drag)
	#event.drag_dest_set(0,[],0)

	top.stick()
	top.set_keep_above(True)
	top.show_all()

	gtk.main()

