_copyright="""
Copyright 2007 Tasci Synx

    This file is part of imageTagger.

    imageTagger is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    imageTagger is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import config
import myurl
from spool import Spool
spool = Spool()

from twisted.web import proxy, http
from twisted.internet import reactor
from twisted.python import log

import sys
import os,tempfile,shutil
import urlparse

import isImage

def is_image(s):
	if s.find('thumb') != -1: return False
	return isImage.is_image(s)

#log.startLogging(sys.stdout)

class ProxyClient(proxy.ProxyClient):
	def __init__(self, command, rest, version, headers, data, father):
		proxy.ProxyClient.__init__(self, command, rest, version, headers, data, father)
		self.type = None
		self.content = None
		self.version = version #grr
	def connectionMade(self):
		# Copied from parent class, because they suck
		self.transport.write('%s %s %s\r\n' % (self.command, self.rest, self.version))
		for header, value in self.headers.items():
			self.sendHeader(header, value)
		self.endHeaders()
		self.transport.write(self.data)
	def handleHeader(self, key, value):
		if key.lower() == 'content-type':
			if value.startswith('image/'):
				self.type = value
				self.content = tempfile.NamedTemporaryFile(dir=config.subfile('tmp'))
				self.handleResponsePart = self.appendContent
				self.handleResponseEnd = self.notifyFather
		proxy.ProxyClient.handleHeader(self,key,value)
	def appendContent(self, buffer):
		proxy.ProxyClient.handleResponsePart(self,buffer)
		if not self.content: return
		self.content.write(buffer)
	def notifyFather(self):
		proxy.ProxyClient.handleResponseEnd(self)
		if not self.content: return
		if self.type:
			length = self.content.tell()
			self.content.seek(0,0)
			self.father.takeImage(self.type,length,self.content)
			self.content = None

class ProxyClientFactory(proxy.ProxyClientFactory):
	protocol = ProxyClient

class ProxyRequest(proxy.ProxyRequest):
	protocols = {'http': ProxyClientFactory}
	# Comment these out to not use a proxy
	proxyProto = 'http'
	proxyHost = 'localhost'
	proxyPort = 3128
	def takeImage(self,type,length,content):
		# Don't care about small images... how to not care about 
		# amazonimages though?
		if length < 2048: return

		# Don't care about thumbnails either
		if not is_image(self.uri): return

		url = myurl.URL(self.uri)
		dest = os.path.basename(url.path)
		spool.dump(self.uri,type,content)
		print "type",type,"length",length
		print "Okay... we saved %s" % self.uri
	def process(self):
		parsed = urlparse.urlparse(self.uri)
		# We should parse it ourselves...
		protocol = parsed[0]
		host = parsed[1]
		port = self.ports[protocol]
		if ':' in host:
			host, port = host.split(':')
			port = int(port)
		rest = urlparse.urlunparse(('','')+parsed[2:])
		if not rest:
			rest = rest+'/'

		realHost = host
		if hasattr(self,'proxyHost'):
			# No, let the upstream proxy handle this
			rest = self.uri
			host = self.proxyHost
			port = self.proxyPort
			protocol = self.proxyProto
			self.setHost(host,port)
		class_ = self.protocols[protocol]
		headers = self.getAllHeaders().copy()
		if not headers.has_key('host'):
			headers['host'] = realHost
		if headers.has_key('connection'):
			del headers['connection']
		self.content.seek(0, 0)
		s = self.content.read()
		clientFactory = class_(self.method, rest, self.clientproto, headers,
							   s, self)
		reactor.connectTCP(host, port, clientFactory)

class Proxy(proxy.Proxy):
	requestFactory = ProxyRequest

class ProxyFactory(http.HTTPFactory):
	protocol = Proxy
 
print "Listening on",reactor.listenTCP(32323, ProxyFactory())._realPortNumber
reactor.run()

