_copyright="""
Copyright 2007 Tasci Synx

    This file is part of imageTagger.

    imageTagger is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    imageTagger is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import gtk
from gtk import glade
import gobject

import random
import os,signal
from subprocess import Popen,PIPE
from operator import attrgetter

import fixbadnames
from resources import resource

import database
import myurl
import taggers

import Image
import myImagetypes

exportLocation = '/var/tmp/tagger'
try:
	os.makedirs(exportLocation)
except OSError: pass

def cleanOut(widget):
    widget.foreach(lambda child: widget.remove(child))

def loadThumb(image):
	loader = gtk.gdk.PixbufLoader('jpeg')
	loader.write(image.thumbnail)
	loader.close()
	return loader.get_pixbuf()

RMIME = dict((v,n.lower()) for n,v in Image.MIME.items())

#class LoadImage:
#	def __init__(self,image,onUpdate):
#		self.closed = False
#		self.data = image.data
#		self.onUpdate = onUpdate
#		loader = gtk.gdk.PixbufLoader(RMIME.get(image.type,'boob'))
#		loader.connect('area-updated',self.updated)
#		self.writeSome(loader)
#	def writeSome(self,loader):
#		if self.closed: return
#		loader.write(self.data[:1024])
#		if len(self.data)>=1024:
#			self.data = self.data[1024:]
#		else:
#			loader.close()
#			self.closed = True
#	def updated(self,loader,*args):
#		self.onUpdate(loader.get_pixbuf())
#		self.writeSome(loader)
#

class TagUpdate:
	_fields_ = (
		'dialog',
		'tagsEntry')
	def __init__(self,db,image,parent,mainUI):
		self.xml = glade.XML(resource("tagupdate.glade"),'dialog')
		for name in self._fields_:
			widget = self.xml.get_widget(name)
			assert widget,"%s not found!" % name
			setattr(self,name,widget)
		self.db = db
		self.image = image
		self.mainUI = mainUI
		self.tagsEntry.set_text(', '.join((tag.name for tag in image.tags)))
		self.tagsEntry.connect("activate",lambda *args: self.commitNewTags())
	
		self.dialog.set_transient_for(parent.dialog)
		self.dialog.set_destroy_with_parent(True)
		self.dialog.show_all()
	def commitNewTags(self):
		tags = set([tag.strip() for tag in self.tagsEntry.get_text().split(',')])
		retags = list()
		for name in tags:
			tag = self.db.findTag(name)
			retags.append(tag)
		retags = set(retags)
		oldTags = set(self.image.tags)
		oldTags.difference_update(retags)
		self.image.setTags(self.db,retags)
		self.db.cleanTags(oldTags)

		self.dialog.destroy()
		self.mainUI.populateTags()

class DisplayImage:
	_fields_ = (
			'dialog',
			'imageArea',
			'exportBtn',
			'tagsBtn',
			'deleteBtn',
			'closeBtn')
	def __init__(self,db,image,mainUI):
		self.xml = glade.XML(resource("image.glade"),'dialog')
		for name in self._fields_:
			widget = self.xml.get_widget(name)
			assert widget,"%s not found!" % name
			setattr(self,name,widget)
		self.dialog.set_title(image.origin)
		self.image = image
		self.db = db
		self.closeBtn.connect('clicked',lambda btn: self.dialog.destroy())
		self.deleteBtn.connect('clicked',lambda btn: self.deleteFromDB())
		self.tagsBtn.connect('clicked',lambda btn: self.createTagUpdate())
		self.exportBtn.connect('clicked',lambda btn: self.doExport())
		self.LoadImage()
		self.mainUI = mainUI
		self.dialog.show_all()
	def doExport(self):
		url = myurl.URL(self.image.origin)
		dest = fixbadnames.fix(os.path.basename(url.path))
		dest = os.path.join(exportLocation,dest)
		out = open(dest,'wb')
		try:
			out.write(self.image.data)
		finally:
			out.close()
		taggers.export(dest,(tag.name for tag in self.image.tags))
		clipboard = Popen(['xclip','-selection','clipboard','-i'],stdin=PIPE)
		clipboard.stdin.write(dest)
		clipboard.stdin.close()
		dialog = gtk.Dialog("Exported",
			None,
			0,
			(gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
		dialog.vbox.pack_start(gtk.Label("The file\n%s\n...has been exported, and the location copied to the clipboard." % dest))
		dialog.vbox.show_all()
		dialog.run()
		dialog.destroy()
		os.kill(clipboard.pid,signal.SIGTERM)
		clipboard.wait()
	def LoadImage(self):
		loader = gtk.gdk.PixbufLoader(RMIME.get(self.image.type,'boob'))
		loader.write(self.image.data)
		loader.close()
		anim = loader.get_animation()
		if anim:
			self.fillImageAnimate(anim.get_iter())
			return
		self.fillImageArea(loader.get_pixbuf())
	def createTagUpdate(self):
		TagUpdate(self.db,self.image,self,self.mainUI)
	def fillImageArea(self,pixbuf):
		self.imageArea.set_from_pixbuf(pixbuf)
	def fillImageAnimate(self,iter):
		self.fillImageArea(iter.get_pixbuf())
		iter.advance()
		gobject.timeout_add(iter.get_delay_time(),self.fillImageAnimate,iter)
	def deleteFromDB(self):
		print "Deleting",self.image.origin
		self.db.session.delete(self.image)
		self.db.session.flush()

class Search:
	_fields_ = (
			'top',
			'notebook',
			'tagList',
			'thumbnailList',
			'selectedBox')
	def __init__(self,db,initialTags=()):
		self.destroyed = False

		self.xml = glade.XML(resource("select.glade"),'top')
		self.db = db
		for name in self._fields_:
			setattr(self,name,self.xml.get_widget(name))

		self.tags = gtk.ListStore(str)
		self.tagList.set_model(self.tags)
		column = gtk.TreeViewColumn("Tags")
		cell = gtk.CellRendererText()
		column.pack_start(cell)
		self.tagList.append_column(column)
		column.set_attributes(cell,text=0)

		self.thumbnails = gtk.ListStore(gtk.gdk.Pixbuf,object)
		self.thumbnailList.set_model(self.thumbnails)
		self.thumbnailList.set_pixbuf_column(0)

		def activateThumbnail(view,path):
			num = path[0]
			image = self.thumbnails[num][1]
			DisplayImage(self.db,image,self)
		self.thumbnailList.connect('item-activated',activateThumbnail)

		self.selectedTags = set(initialTags)
		def narrowSelection(view,num,column):
			num = num[0]
			self.selectedTags.add(self.tags[num][0])
			self.populateTags()
		self.tagList.connect("row-activated",narrowSelection)
		self.populateTags()

		self.top.connect('destroy',lambda *args: gtk.main_quit())
		self.top.maximize()
		self.top.show_all()
	def removeSelected(self,tag):
		self.selectedTags.remove(tag)
		self.populateTags()
	def fillSelectedBox(self):
		cleanOut(self.selectedBox)
		def callbackForTag(tag):
			def callback(btn):
				self.removeSelected(tag)
			return callback
		for tag in self.selectedTags:
			button = gtk.Button(label=tag)
			self.selectedBox.pack_start(button)
			button.connect('clicked',callbackForTag(tag))
		self.selectedBox.show_all()
	def populateTags(self):
		self.fillSelectedBox()
		self.tags.clear()
		selectedTags = tuple(self.selectedTags)
		if len(selectedTags):
			images = self.db.findTag(selectedTags[0]).images
		else:
			query = self.db.session.query(database.Image)
			images = list(query.select())
		for tag in selectedTags[1:]:
			reimages = list()
			for image in images:
				for t in image.tags:
					if tag == t.name:
						reimages.append(image)
			images = reimages
		retags = set()
		random.shuffle(images)
		for image in images:
			for tag in image.tags:
				if tag.name in self.selectedTags: continue
				retags.add(tag)
		retags = list(retags)
		retags.sort(key=attrgetter('name'))
		for tag in retags:
			self.tags.append([tag.name])
		images = images[:20]
		self.thumbnails.clear()
		for image in images:
			self.thumbnails.append([loadThumb(image),image])
	def destroy(self):
		if self.destroyed: return
		self.notebook.destroy()
		for attr in self._fields_:
			delattr(self,attr)
		self.destroyed = True
	def __del__(self):
		self.destroy()

def test(where,tags=()):
	gtk.window_set_default_icon_from_file(resource('icon.png'))
	if not where.startswith('sqlite'):
		where = 'sqlite:///'+where
	db = database.open(where)
	app = Search(db,tags)
	gtk.main()

if __name__=='__main__': 
	import sys
	try:
		test(sys.argv[1],sys.argv[2].split(','))
	except IndexError:
		test(sys.argv[1])
